"""
Old, text-bases annotations.  Superceded by dbannotations.

This now lacks a couple of features required by annotate and friends,
but it wouldn't be hard to retrofit them.
"""

import os

from ianno import common

class AnnotationCollection(object):
	"""is a container for image annotations.

	Basically, we store an absolute path as key, some string, and
	possibly tags.
	"""
	def __init__(self, initAnns=None):
		self.data = {}
		if initAnns:
			for ann in initAnns:
				self.add(ann)

	def add(self, ann):
		self.data[ann.key] = ann
	
	def delete(self, key):
		if key in self.data:
			del self.data[key]

	def keys(self):
		return self.data.keys()

	def get(self, key):
		return self.data.get(key)

	def getTagFreqs(self):
		freqs = {}
		for ann in self.data.values():
			for tag in ann.tags:
				freqs[tag] = freqs.get(tag, 0)+1
		return freqs

	def getSubCollection(self, tags):
		"""returns a subset of AnnotationCollection containing only
		annotations tagged with all of the set tags.
		"""
		res = []
		for ann in self.data.values():
			if tags&ann.tags==tags:
				res.append(ann)
		return AnnotationCollection(res)


class Annotations(AnnotationCollection):
	"""is the user-global annotation collection.

	It is filled on construction for a hidden file in the user's home.
	"""

	storageName = os.path.join(os.environ["HOME"], ".iannontations")
	badStorageName = os.path.join(os.environ["HOME"], ".iannontations-bad")

	def __init__(self):
		AnnotationCollection.__init__(self)
		self._load()

	def _load(self):
		if os.path.exists(self.storageName):
			f = open(self.storageName)
			for ln in f:
				self.addSerializedAnnotation(ln[:-1])
			f.close()

	def save(self):
		tmpName = self.storageName+".tmp"
		f = open(tmpName, "w")
		for ann in self.data.values():
			f.write(ann.toString()+"\n")
		f.close()
		os.rename(tmpName, self.storageName)

	def _swallowBadFormat(self, badLine):
		f = open(self.badStorageName, "a")
		f.write(badLine)
		f.close()

	def addSerializedAnnotation(self, sourceLine):
		try:
			self.add(common.Annotation.fromString(sourceLine))
		except common.FormatError:
			self._swallowBadFormat(sourceLine)
